/* mwcollect2 (c) 2004-2005 Georg Wicherski
 * this code is licensed under the mwcollect2 license, the the file LICENSE for details
 * distribution of this code without written permission strictly prohibited
 *
 * modules.cpp - implementation of ModuleRepresentation
 *
 * 2005-01-05	 fast rewrite
 */

#include <mwcollectd.h>
#include <stdlib.h>
#include <errno.h>

namespace mwcollectd
{
	ModuleRepresentation::ModuleRepresentation(const char * szName, mwcollectd::LogManager * pLogger)
	{
		m_pLogger = pLogger;
		m_szModuleName = strdup(szName);

		m_pModule = 0;
		m_pLibrary = 0;
	}

	ModuleRepresentation::~ModuleRepresentation()
	{
		if(m_szModuleName)
			free(m_szModuleName);
	}

	bool ModuleRepresentation::loadLibrary(const char * szFileName)
	{ // loads the .dll/.so associated with a module

		#if defined(WIN32)
			m_pLibrary = LoadLibrary(szFileName);
		#elif defined(LINUX)
			m_pLibrary = dlopen(szFileName, RTLD_NOW);
		#endif

		if(!m_pLibrary)
		{
			m_pLogger->log(LL_CRITICAL, "Could not load module %s (%s)!", m_szModuleName, szFileName);
			 m_pLogger->log(LL_CRITICAL,"Reason: %s",dlerror ());
			return false;
		}

		#if defined(WIN32)
			(FARPROC&) m_fnCreateInstance = GetProcAddress(m_pLibrary, "CreateInstance");
		#elif defined(LINUX)
			m_fnCreateInstance = (module_facility_call_t) dlsym(m_pLibrary, "CreateInstance");
		#endif

		if(!m_fnCreateInstance)
		{
			m_pLogger->log(LL_CRITICAL, "%s does not provide symbol \"CreateInstance\"!", szFileName);

			return false;
		}		

		m_pLogger->log(LL_DEBUG, "Loaded module %s (%s).", m_szModuleName, szFileName);

		return true;
	}

	bool ModuleRepresentation::freeLibrary()
	{ // frees the associated library
		if(!m_pLibrary)
			return false;

		m_pLogger->log(LL_DEBUG, "Unloading %s...", m_szModuleName);

		if(m_pModule)
			delete m_pModule;

		#if defined(WIN32)
			FreeLibrary(m_pLibrary);
		#elif defined(LINUX)
			dlclose(m_pLibrary);
		#endif

		m_pLibrary = 0;

		return true;
	}

	bool ModuleRepresentation::initialiseModule(mwcollectd::MalwareCollector * pCollector, mwcollectd::LogManager* pLogger, Configuration * pConfig)
	{ // initializes module, makes m_pModule available for further ::start(), ::loop() and ::stop()
		if(!(m_pModule = m_fnCreateInstance(pCollector)))
		{
			m_pLogger->log(LL_CRITICAL, "CreateInstance for %s failed!", m_szModuleName);

			return false;
		}

		m_pModule->setLogManager(pLogger);

		{
			char * szPath;

			asprintf(&szPath, ":modules:%s", m_szModuleName);
			m_pModule->setConfiguration(pConfig, szPath);
			free(szPath);
		}

		return true;
	}



	bool MalwareCollector::loadModules()
	{
		std::list<const char *> * plszModules = m_pConfiguration->getChildren(":modules");

		if(!plszModules)
			return false;

		for(std::list<const char *>::iterator i = plszModules->begin(); i != plszModules->end(); ++i)
			loadSingleModule(* i); // failure to load a single module doesn't yield to exit,

		return true;
	}

	bool MalwareCollector::loadSingleModule(const char * szName)
	{
		char * szPath;
		ModuleRepresentation mModule = ModuleRepresentation(szName, m_pLogManager);

		{
			asprintf(&szPath, ":modules:%s:library", szName);			

			if(!m_pConfiguration->leafExists(szPath))
			{
				m_pLogManager->log(LL_CRITICAL, "Module Block for %s does not contain the value 'library'.", szName);

				return false;
			}

			if(!mModule.loadLibrary(m_pConfiguration->getString(szPath, 0)))
				return false;

			free(szPath);
		}		

		if(!mModule.initialiseModule(this, m_pLogManager, m_pConfiguration))
			return false;

		m_lmModules.push_back(mModule);
		return true;
	}
}
