/* mwcollect2 (c) 2004-2005 Honeynet Project
 * this code is licensed under the BSD License, the the file LICENSE for details
 *
 * mwcollectd.cpp - initialization and lovely heart of the mwcollectd daemon
 *
 * 2004-12-28	 basic startup routines
 */

#include <mwcollectd.h>
#include "./config.h"

#include <unistd.h>
#include <getopt.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/resource.h>

namespace mwcollectd
{
	bool g_fRun; // only used by SIGINT handler to tell the MalwareCollector to stop its freaking business

	void interrupt_handler(int i)
	{
		g_fRun = false;
	}

	int MalwareCollector::run(int iArgC, char * * szArgV)
	{ // our actual initialization routine, not thread safe (-> getopt)
		const char * szConfigFile = DEFAULT_CONFIG_FILE;
		const char * szLogLevel = 0;
		bool fDaemon = false;
		char cChar;

		LogManager * pLogManager = new LogManager(DEFAULT_LOG_FILTER);
		Configuration * pConfig = new Configuration();		
		
		{ // parsing the command line using POSIX provided getopt, also detaching from console using POSIX fork()
			opterr = 0;

			while((cChar = getopt(iArgC, szArgV, "c:l:L:DhCv")) != -1)
			{
				switch(cChar)
				{
				case 'c':
					szConfigFile = optarg;
					break;

				case 'l':
				{ // if there hase been some -L adopt it, else it's DEFAULT_LOG_FILTER
					// this works a bit like a state machine (see example in usage)
					FileLogger * pLogger = new FileLogger(optarg, LL_NONE);						

					pLogManager->addLogger(pLogger);

					if(szLogLevel)
						pLogger->setLogLevel(LogManager::mapLogLevel(szLogLevel));

					break;
				}

				case 'C':
				{ // the console is a file as well
					FileLogger * pLogger = new FileLogger(stdout, LL_NONE);

					pLogManager->addLogger(pLogger);

					if(szLogLevel)
						pLogger->setLogLevel(LogManager::mapLogLevel(szLogLevel));
				}
					
				break;

				case 'D':
					fDaemon = true;
					break;

				case 'L':
					szLogLevel = optarg;
					break;

				case '?':
				case 'h':
				default:
					fprintf(stderr, "Usage: %s [options]\r\nWhere options can be one of the following:\r\n	-c <config-file>\tspecify config file (defaults to " DEFAULT_CONFIG_FILE ")\r\n	-l <log-file>\t\tadd log file\r\n	-C\t\t\tadd console logger\r\n	-L <log-level>\tspecify log level, inital is %u\r\n	-D\t\t\tdetach from console\r\n	-h\t\t\tdisplay this info\r\n\r\nExample:\r\n\t%s -L critical -C -L info -l /var/log/mwcollectd.log -c /etc/mwcollectd.conf\r\n\r\n\r\n", szArgV[0], DEFAULT_LOG_FILTER, szArgV[0]);
	
				case 'v':
					fprintf(stderr, "mwcollect2 (c) 2004- 2005 Honeynet Project\r\nBased on a collaboration of Paul Bcher, Thorsten Holz, Markus Ktter and Georg Wicherski.\r\n");
					return 0;
				}
			}
	
			if(!pConfig->parse(szConfigFile))
			{
				pLogManager->log(LL_CRITICAL, "Could not parse configuration file!");

				return 1;
			}

			if(fDaemon && fork())
			{
				pLogManager->log(LL_DEBUG, "Detached from console.");

				return 0;
			}

			signal(SIGINT, interrupt_handler);

			{
				unsigned int nBlocks, nValues, nCharacters;
				
				pLogManager->log(LL_INFO, "Starting mwcollectd2, have fun!");

				pConfig->statistics(nBlocks, nValues, nCharacters);
				pLogManager->log(LL_DEBUG, "Configuration file has %u non-root blocks, %u values and %u total characters.", nBlocks, nValues, nCharacters);
			}
		}

		{ // creating the actual MalwareCollector object
			MalwareCollector * pCollector;

			g_fRun = true;
			pCollector = new MalwareCollector();

			pCollector->setLogManager(pLogManager);
			pCollector->setConfiguration(pConfig);

			pCollector->start();

			while(pCollector->active() && g_fRun)
				pCollector->loop();

			pCollector->stop();
		}

		{ // clean up			
			delete pConfig;
			delete pLogManager;
		}

		return 0;
	}


	
	MalwareCollector::MalwareCollector()
	{ // setup
		m_pNetworkInterface = 0;

		m_pSubmissionDispatcher = 0;
		m_pFetchDispatcher = 0;
		m_pShellcodeDispatcher = 0;

		m_pShell = 0;
	}

	MalwareCollector::~MalwareCollector()
	{ // nothing to destruct yet
	}

	
	void MalwareCollector::start()
	{ // start 'collecting'	
		{ // increase fd limit (high traffic collectors)
		
			if(!geteuid()) // wow, we're root
			{
				struct rlimit lmLimit;
				long lAdjust = m_pConfiguration->getLong("max-filedescriptors", 0);
				
				if(lAdjust)
				{
					lmLimit.rlim_cur = lAdjust;
					lmLimit.rlim_max = lmLimit.rlim_cur;
					
					if(setrlimit(RLIMIT_NOFILE, &lmLimit) != 0)
						m_pLogManager->log(LL_CRITICAL, "Could not change max filedescriptors limit!");
					else
						m_pLogManager->log(LL_DEBUG, "Set max filedescriptors limit to %u.", lAdjust);
				}
			}		
		}
		
		{ // start myself
			m_bActive = false;

			m_pNetworkInterface = new NetworkInterface(m_pLogManager, m_pConfiguration);

			m_pSubmissionDispatcher = new SubmissionDispatcher(m_pLogManager, m_pConfiguration);
			m_pFetchDispatcher = new FetchDispatcher(m_pSubmissionDispatcher, m_pLogManager, m_pConfiguration);
			m_pShellcodeDispatcher = new ShellcodeDispatcher(m_pLogManager);

			m_pShell = new ShellEmulation(this);
			m_pShell->setLogManager(m_pLogManager);
			m_pShell->setConfiguration(m_pConfiguration);
			m_pShell->start();

			if(!loadModules())
			{
				m_pLogManager->log(LL_CRITICAL, "Could not load Modules!");

				return;
			}

			m_bActive = true;
		}

		{ // launch all the modules
			for(std::list<ModuleRepresentation>::iterator i = m_lmModules.begin(); i != m_lmModules.end(); ++i)
				(* i).getModule()->start();
		}
	}

	void MalwareCollector::loop()
	{ // do one iteration
		{ // my stuff
			m_pNetworkInterface->loop();

			m_pShell->loop();
		}

		{ // the modules
			for(std::list<ModuleRepresentation>::iterator i = m_lmModules.begin(); i != m_lmModules.end(); ++i)
				(* i).getModule()->loop();
		}
	}

	void MalwareCollector::stop()
	{ // stop 'collecting'
		{ // stop all the modules
			for(std::list<ModuleRepresentation>::iterator i = m_lmModules.begin(); i != m_lmModules.end(); ++i)
				(* i).getModule()->stop();
		}

		m_pLogManager->log(LL_DEBUG, "MalwareCollector::stop()");

		{ // clean myself
			m_bActive = false;

			delete m_pNetworkInterface;

			m_pShell->stop();
			delete m_pShell;

			m_pLogManager->log(LL_INFO, "Graceful shutdown of mwcollectd2.");	
		}
	}

	bool MalwareCollector::active()
	{ // still active? this could be useful if there come more abort situations than ::stop()
		return m_bActive;
	}


	void MalwareCollector::setConfiguration(Configuration * pConfig)
	{ // set our configuration interface
		m_pConfiguration = pConfig;
	}

	void MalwareCollector::setLogManager(LogManager * pManager)
	{ // set the log manager we log to
		m_pLogManager = pManager;
	}


	NetworkInterface * MalwareCollector::getNetworkInterface()
	{
		return m_pNetworkInterface;
	}

	SubmissionDispatcher * MalwareCollector::getSubmissionDispatcher()
	{
		return m_pSubmissionDispatcher;
	}

	FetchDispatcher * MalwareCollector::getFetchDispatcher()
	{
		return m_pFetchDispatcher;
	}

	// MalwareCollector::loadLibraries et al. are in modules.
}
